=begin
###########################################################################
TIG (c) 2010
All Rights Reserved.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
WARRANTIES,INCLUDING,WITHOUT LIMITATION,THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
###########################################################################
  materialconsolidator.rb
###########################################################################
Type 'MaterialConsolidator.new' in the Ruby Console.
OR
'Menu' > 'Tools' > 'Material Consolidator'
All materials with the same 'properties' are consolidated into one, and the 
rest are purged... [the progress is reported in the Console/Command-line]
It is one step undoable.
###########################################################################
Donations:
  By Paypal.com to info @ revitrev.org
###########################################################################
Version:
1.0 20101218 First issue.
1.1 20101218 Menu added.
1.2 20101219 Listing/Sorting optimized, command-line prompts added.
1.3 20110116 Wrapped into a class 'MaterialConsolidator'.
###########################################################################
=end
require 'sketchup.rb'
###
class MaterialConsolidator
def initialize()
    model=Sketchup.active_model
    begin
      model.start_operation("MaterialConsolidator", true)
    rescue
      model.start_operation("MaterialConsolidator")
    end
    msg="Material Consolidator: Making Material Lists"
    Sketchup.set_status_text(msg)
    puts msg
    ents=model.entities
    defs=model.definitions
    mats=model.materials.to_a
    dups=[]
    while mats[0]
      Sketchup.set_status_text(msg+".")
      m0=mats[0]
      dup=[m0]
      mats.dup.each{|mat|
        next if mat.materialType != m0.materialType
        next if mat.color.red != m0.color.red
        next if mat.color.green != m0.color.green
        next if mat.color.blue != m0.color.blue
        next if mat.use_alpha? != m0.use_alpha?
        next if mat.alpha != m0.alpha
        next if not mat.texture and m0.texture
        next if mat.texture and not m0.texture
        if mat.texture and m0.texture
          mtex=mat.texture; itex=m0.texture
          next if mtex.filename != itex.filename
          next if mtex.image_width != itex.image_width
          next if mtex.image_height != itex.image_height
          next if mtex.width != itex.width
          next if mtex.height != itex.height
          dup << mat
          mats.delete(mat)
        else ### neither have textures
          dup << mat
          mats.delete(mat)
        end#if
      }
      dups << dup
      mats.delete(m0)
    end#while
    ### get a list of objects with materials once
    msg="Material Consolidator: Making Entities List"
    Sketchup.set_status_text(msg)
    puts msg
    all=[]
    ents.each{|e|
        Sketchup.set_status_text(msg+".")
        if e.respond_to?(:material) and e.material
            all << e
        end#if
        if e.respond_to?(:back_material) and e.back_material and not all.include?(e)
            all << e
        end#if
    }
    msg="Material Consolidator: Making Nested Entities List"
    Sketchup.set_status_text(msg)
    puts msg
    defs.each{|defn|
        next if defn.image?
        Sketchup.set_status_text(msg+".")
        defn.entities.to_a.each{|e|
            if e.respond_to?(:material) and e.material
                all << e
            end#if
            if e.respond_to?(:back_material) and e.back_material and not all.include?(e)
                all << e
            end#if
        }
    }
    ###
    msg="Material Consolidator: Consolidating Materials"
    Sketchup.set_status_text(msg)
    puts msg
    all.each{|e|
        Sketchup.set_status_text(msg+".")
        dups.each{|dup|
            next if not dup[1]
            mat=dup[0]
            next if not mat.valid?
            dup[1..-1].each{|m|
                next if not m.valid?
                if e.material==m ### all will
                    e.material=mat
                end#if then for faces only...
                if e.respond_to?(:back_material) and e.back_material==m
                    e.back_material=mat
                end#if
            }  
        }
    }
    ### purge unused
    msg="Material Consolidator: Purging Duplicated Materials"
    Sketchup.set_status_text(msg)
    puts msg
    mats=model.materials
    mats.current=nil
    count0=mats.to_a.length
    mats.purge_unused
    count1=mats.to_a.length
    count=count0-count1
    msg="Material Consolidator: "+count.to_s+" Materials Purged"
    Sketchup.set_status_text(msg)
    puts msg
    ###
    model.commit_operation
    ###
end
end#class
###########################################################################
### menu
if not file_loaded?(__FILE__)
    UI.menu("Tools").add_item("Material Consolidator"){MaterialConsolidator.new()}
end#if
file_loaded(__FILE__)
###########################################################################
